import 'dart:io';

import 'package:args/args.dart';
import 'package:flavored_env/src/config.dart';
import 'package:flavored_env/src/models/flavored_env.dart';
import 'package:flavored_env/src/version.dart';
import 'package:path/path.dart';
import 'package:pub_updater/pub_updater.dart';
import 'package:yaml/yaml.dart';

class FlavoredEnvCommandRunner {
  FlavoredEnvCommandRunner({
    PubUpdater? pubUpdater,
  }) : _pubUpdater = pubUpdater ?? PubUpdater();

  late final PubUpdater _pubUpdater;

  Future<void> run(List<String> arguments) async {
    // parser definition
    final parser = ArgParser()
      ..addFlag(
        'version',
        abbr: 'v',
        negatable: false,
        help: 'Print the current version.',
      )
      ..addFlag(
        'help',
        abbr: 'h',
        negatable: false,
        help: 'Print this usage information.',
      )
      ..addFlag(
        cmdGenerate,
        abbr: cmdGenerate[0],
        negatable: false,
        help: 'Generate the requested output from $kTemplateName.',
      )
      ..addFlag(
        cmdCreate,
        abbr: cmdCreate[0],
        negatable: false,
        help: 'Create a template yaml file to get started.',
      );

    // parse the command line parameters
    final results = parser.parse(arguments);

    // help requested
    if (results.wasParsed('help') || results.options.isEmpty) {
      stdout.writeln(parser.usage);
      exit(0);
    } else if (results.wasParsed('version')) {
      stdout.writeln('flavored_env version: $packageVersion');
      exit(0);
    }
    if (results.wasParsed('create-yaml')) {
      try {
        final File file = File(join(Directory.current.path, kTemplateName));
        if (file.existsSync()) {
          stdout.writeln('$kYamlTemplate already exists, no action');
          exit(-1);
        }
        file.writeAsStringSync(kYamlTemplate);
      } catch (e) {
        stdout.writeln(e);
        exit(-1);
      }
    } else if (results.wasParsed(cmdGenerate)) {
      try {
        final File file = File(join(Directory.current.path, kTemplateName));
        if (!file.existsSync()) {
          throw Exception('No $kTemplateName file');
        }
        // parse the yaml fie
        final YamlMap yamlMap = loadYaml(file.readAsStringSync()) as YamlMap;
        // generate the output
        FlavoredEnv.fromYamlMap(yamlMap)
          ..generateEnv()
          ..generateDart();
      } catch (e) {
        stdout.writeln('Error parsing $kTemplateName - $e');
        exit(-1);
      }
    }

    await _checkForUpdates();

    exit(0);
  }

  Future<void> _checkForUpdates() async {
    try {
      final latestVersion = await _pubUpdater.getLatestVersion(packageName);
      final isUpToDate = packageVersion == latestVersion;
      if (!isUpToDate) {
        stdout.writeln(
          '''
Update available!  $packageVersion --> $latestVersion
Changelog:' https://github.com/berbsd/flavored_env/releases/tag/v$latestVersion')}
Run 'dart pub global activate flavored_env' to update''',
        );
      }
    } catch (_) {}
  }
}
